<?php
namespace Dugi;

use WPCF7_Submission;

/**
 * @author  Dagan Yaakov <admin@dugi.co.il>
 * @since   08/12/2016
 * @update  03/12/2019
 * @link    http://wp.dugi.co.il
 * @version 1.1.0
 */


class leadmanager_cf7_driver{
	
	protected $enable;
	protected $skip_mail;
	
	public function __construct(){
		global $leadmanager_options;
		$this->enable = (bool)$leadmanager_options->get('inc_cf7_module');
		$this->skip_mail = (bool)$leadmanager_options->get('cf7_skip_mail');
		
		add_action( 'wpcf7_before_send_mail', array($this,'wpcf7_before_send_mail'),1);
		
		$fn_lm_content = function(){
			if (!empty($_SESSION['lm_content'])){
				
				if( !session_id() ) session_start();
				$lm_content = $_SESSION['lm_content'];
				$_SESSION['lm_content'] = null;
				header('Content-Type: application/javascript');
				echo $lm_content;
				exit;
			}
				
			
		};
		add_action("wp_ajax_lm_content.js",$fn_lm_content);
		add_action("wp_ajax_nopriv_lm_content.js",$fn_lm_content);
		add_action('wp_footer',function(){
			?>
			<script>
			document.addEventListener('wpcf7submit', function( event ) { 
				var script = document.createElement('script');
				script.src = '<?php echo admin_url( 'admin-ajax.php?action=lm_content.js' ); ?>';
				document.head.appendChild(script);
			}, false );
			</script>
			<?php
		});
	}
	
	 public function convertData($cf7) {
        if (!isset($cf7->posted_data) && class_exists('WPCF7_Submission')) {
            // Contact Form 7 version 3.9 removed $cf7->posted_data and now
            $submission = WPCF7_Submission::get_instance();
            if ($submission) {
                $data = array();
                $data['title'] = $cf7->title();
                $data['posted_data'] = $submission->get_posted_data();
                $data['uploaded_files'] = $submission->uploaded_files();
                $data['WPCF7_ContactForm'] = $cf7;

                $data['posted_data']['Page Title'] = wpcf7_special_mail_tag('', '_post_title', '');
                $data['posted_data']['Page URL'] = wpcf7_special_mail_tag('', '_post_url', '');

                return (object) $data;
            }
        }
        return $cf7;
    }
	
	public function wpcf7_before_send_mail($cf7){
		global $leadmanager_logger,$leadmanager_sender;
		$data = $this->convertData($cf7);
		$posted_data = (isset($data->posted_data)) ? $data->posted_data : array();
		
		if ($this->skip_mail){
			$cf7->skip_mail = true;
		}
		/*
		if (isset($data['your-recipient']) && is_email($data['your-recipient'])){
			//$wpcf7->skip_mail = null;
		}elseif (isset($data['alsosendemail'])){
			//$wpcf7->skip_mail = null;
		}else{
			$wpcf7->skip_mail = true;
		}	
		*/
		
		if (isset($posted_data['lm_post_method']) && ($posted_data['lm_post_method'] == 'lm_proxy' || $posted_data['lm_post_method'] == 'lm_iframe' || $_POST['lm_post_method'] == 'lm_pixel' || $_POST['lm_post_method'] == 'lm')){
			$posted_data = wp_parse_args( leadmanager_session::get_protected_args(),(array)$posted_data );
			$leadmanager_sender->setData($posted_data);
			$location_url = $leadmanager_sender->send();
			
			$cf7_properties = $cf7->get_properties();
			$backup_additional_settings = isset($cf7_properties['additional_settings']) ? $cf7_properties['additional_settings'] : '';
			
			
			if ($posted_data['lm_post_method'] == 'lm' && $location_url){
				$_SESSION['lm_content'] = "location.replace('{$location_url}');";
				$cf7->set_properties( array(
					'additional_settings' => sprintf('on_sent_ok: "location.replace(%1$s);"',"'$location_url'"),
				) );
			}elseif($posted_data['lm_post_method'] == 'lm_iframe' && $location_url){
				$_SESSION['lm_content'] = "!function(){var e=document.createElement('iframe');e.src='{$location_url}',e.width=1,e.height=1,document.body.appendChild(e)}();";
				$cf7->set_properties( array(
					'additional_settings' => sprintf('on_sent_ok: "!function(){var e=document.createElement(\'iframe\');e.src=\'%1$s\',e.width=1,e.height=1,document.body.appendChild(e)}();"',$location_url) . "\r\n" . $backup_additional_settings,
				) );
			}elseif($posted_data['lm_post_method'] == 'lm_proxy' && $location_url){
				$_SESSION['lm_content'] = "!function(){var e=document.createElement('iframe');e.src='{$location_url}',e.width=1,e.height=1,document.body.appendChild(e)}();";
				$cf7->set_properties( array(
					'additional_settings' => sprintf('on_sent_ok: "!function(){var e=document.createElement(\'iframe\');e.src=\'%1$s\',e.width=1,e.height=1,document.body.appendChild(e)}();"',$location_url) . "\r\n" . $backup_additional_settings,
				) );
			}elseif($posted_data['lm_post_method'] == 'lm_pixel' && $location_url){
				$_SESSION['lm_content'] = "!function(){var e=document.createElement('img');e.src='{$location_url}',e.width=1,e.height=1,document.body.appendChild(e)}();";
				$cf7->set_properties( array(
					'additional_settings' => sprintf('on_sent_ok: "!function(){var e=document.createElement(\'img\');e.src=\'%1$s\',e.width=1,e.height=1,document.body.appendChild(e)}();"',$location_url) . "\r\n" . $backup_additional_settings,
				) );
			}else{
				$_SESSION['lm_content'] = null;
			}
			
		}

	}
}