<?php 
namespace Dugi;

/**
 * Usage:
 * $leadmanager_sender = new Dugi\leadmanager_sender('/var/log/');
 * $leadmanager_sender->setData(array);
 * $leadmanager_sender->send();
 * return REDIRECT URL OR FALSE;
 *
 * @author  Dagan Yaakov <admin@dugi.co.il>
 * @since   08/12/2016
 * @link    http://wp.dugi.co.il
 * @version 1.0.0
 */
 
 
class leadmanager_sender{
	protected $options = array (
		'maxAttempts'   		  => 9,
        'timeBetweenAttempts'     => 300,
        'fibonacciAttemptsModel'  => true,
		'minLeadID'				  => 1000,
    );
	
	
	private $endpointURL;
	private $currentAttempts;
	private $data;
	
	
	public function __construct($endpointURL = null, array $options = array()){
        if ($endpointURL === null) 
				return false;
			
		$this->options = array_merge($this->options, $options);
		
		$this->endpointURL = $endpointURL;
		$this->currentAttempts = 0;
		
		add_action('leadmanager_sender_retry',array(__NAMESPACE__ . '\\leadmanager_sender','cronjob'),10,3);
	}
	public function set_currentAttempts($int){
		$this->currentAttempts = absint($int);
	}
	
	public function setData($data){
		$this->data = (array)$data;
	}
	
	private function retry(){
		$this->currentAttempts++;
		
		if ($this->options['fibonacciAttemptsModel']){
			$crontime = absint($this->getFib($this->currentAttempts) * $this->options['timeBetweenAttempts']) + time(); 
		}else{
			$crontime = time() + $this->options['timeBetweenAttempts'];
		}
		wp_schedule_single_event($crontime ,'leadmanager_sender_retry',array($this->endpointURL,$this->data,$this->currentAttempts));
	}
	
	public function send(array $args = array()){
		global $leadmanager_logger,$leadmanager_mailer;
		
		if (!$this->data)
			$this->setData();
		
		
		$is_cron = ($this->currentAttempts) ? true : false;
		if ($this->currentAttempts >= $this->options['maxAttempts']){
			//maxAttempts
			$leadmanager_logger->add("*** MAX Attempts {$this->currentAttempts}/{$this->options['maxAttempts']} ***");
			$leadmanager_logger->add($this->data);
			$leadmanager_logger->save();
		   return false;
		}elseif($this->currentAttempts > 0){
			$leadmanager_logger->add("*** Attempts {$this->currentAttempts}/{$this->options['maxAttempts']} ***");
		}else{
			
		}
		
		
		$defaults = array(
			'method' => 'POST',
			'timeout' => 5,
			'redirection' => 0,
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => array(),
			'body' => $this->data,
			'cookies' => array()
			);
			
		$args = wp_parse_args( $args, $defaults );
		$microtime_start = microtime(true);
		/*---->*/$response = wp_remote_post($this->endpointURL,$args);/*<----*/
		$microtime_total = microtime(true) - $microtime_start;
		
		if ( is_wp_error($response) ) {
		   $error_message = $response->get_error_message();
		   //LOG!
		   $leadmanager_logger->add("Method: {$args['method']} Endpoint: {$this->endpointURL}");
		   $leadmanager_logger->add("Request time: {$microtime_total}");
		   $leadmanager_logger->add("Error: {$error_message}");
		   $leadmanager_logger->add("User Agent: {$_SERVER['HTTP_USER_AGENT']}");
		   $leadmanager_logger->add($this->data);
		   $leadmanager_logger->save();
		   
		   $this->retry();
		   
		   if (!$this->currentAttempts){
			   $leadmanager_mailer->send("LM WORDPRESS: {$error_message}",sprintf(__('[%s] ליד לא נשלח ל LeadManager.co.il'), get_option('blogname')),'ERROR');
		   }
		   return false;
		} else {
			$response_code = absint(wp_remote_retrieve_response_code($response));
			$response_message = wp_remote_retrieve_response_message($response);
			$response_body = wp_remote_retrieve_body($response);
			$lm_lead_id = intval($response_body);
			$location_url = (isset($response['headers']['location'])) ? $response['headers']['location'] : '';
			
			if ($lm_lead_id > absint($this->options['minLeadID'])){
				//All Good!
				$leadmanager_logger->add("Response: {$response_code} {$response_message} [{$_SERVER['REMOTE_ADDR']}]");
				$leadmanager_logger->add("Request time: {$microtime_total}");
				$leadmanager_logger->add("Lead ID: {$lm_lead_id}");
				$leadmanager_logger->save('-');
				return $location_url;
			}elseif($lm_lead_id < 0){
				$leadmanager_logger->add("Response: {$response_code} {$response_message}");
				$leadmanager_logger->add("Request time: {$microtime_total}");
				$leadmanager_logger->add("LM ERROR: {$lm_lead_id}");
				$leadmanager_logger->add($this->data);
				$leadmanager_logger->save();
				//LOG
				//notify to customer
			    $leadmanager_mailer->send("LM WORDPRESS ERROR: {$lm_lead_id}",sprintf(__('[%s] ליד לא נשלח ל LeadManager.co.il'), get_option('blogname')),'NOTICE');

			}elseif($response_code == 500){
				$leadmanager_logger->add("Method: {$args['method']} Endpoint: {$this->endpointURL}");
				$leadmanager_logger->add("Response: {$response_code} {$response_message}");
				$leadmanager_logger->add("Request time: {$microtime_total}");
				$leadmanager_logger->add("User Agent: {$_SERVER['HTTP_USER_AGENT']}");
				$leadmanager_logger->add($this->data);
				$leadmanager_logger->save();
				//notify to customer and support
				$leadmanager_mailer->send("LM WORDPRESS ERROR: {$response_code} {$response_message}",sprintf(__('[%s] ליד לא נשלח ל LeadManager.co.il'), get_option('blogname')),'WARNING');
			}else{
				$leadmanager_logger->add("Method: {$args['method']} Endpoint: {$this->endpointURL}");
				$leadmanager_logger->add("Response: {$response_code} {$response_message}");
				$leadmanager_logger->add("Request time: {$microtime_total}");
				$leadmanager_logger->add("User Agent: {$_SERVER['HTTP_USER_AGENT']}");
				$leadmanager_logger->add($this->data);
				$leadmanager_logger->save();
				//???....
				$leadmanager_mailer->send("LM WORDPRESS ERROR: {$response_code} {$response_message}",sprintf(__('[%s] ליד לא נשלח ל LeadManager.co.il'), get_option('blogname')),'WARNING');

			}
			
			
			
		}
		
		
	}
	
	
	public function getFib($n){
		return round(pow((sqrt(5)+1)/2, $n) / sqrt(5));
	}
	
	static function cronjob($endpointURL,$data,$currentAttempts){
		$instance = new self($endpointURL);
		$instance->set_currentAttempts($currentAttempts);
		$instance->setData($data);
		$instance->send();
	}
	
	
}


		
 