<?php 

class leadmanager_update{
	
	const LM_TRANSIENT_KEY 	 = 'lm_update_json';
	const LM_TRANSIENT_TIME  = 39600;
	const LM_UPDATE_JSON_URL = 'https://leadmanager.co.il/_lm-wordpress/information.json';
	
	public $plugin_file;
	public $plugin_data;
	
	public function __construct($plugin_file){
		
		$this->plugin_file = $plugin_file;
		
		add_filter('plugins_api', [$this,'plugins_api'], 10, 3);
		add_filter('site_transient_update_plugins', [$this,'push_update']);
		add_action( 'upgrader_process_complete', [$this,'after_update'], 10, 2 );
		
		
	}
	
	public function get_plugin_data($key,$def=null){
		
		if (empty($this->plugin_data)){
			$this->plugin_data = get_plugin_data($this->plugin_file);
			$this->plugin_data['LM_PLUGIN'] = ltrim(str_replace(WP_PLUGIN_DIR,'',$this->plugin_file),'/\\');
			$this->plugin_data['LM_PLUGIN_SLUG'] = basename(dirname($this->plugin_file));
		}
		
		return (isset($this->plugin_data[$key])) ? $this->plugin_data[$key] : $def;
	}
	
	public function plugins_api( $res, $action, $args ){
		
		if( $action !== 'plugin_information' )
			return false;
		
		if( $this->get_plugin_data('LM_PLUGIN_SLUG') !== $args->slug )
			return false;
		
		
		

		if( false == $remote = get_transient( self::LM_TRANSIENT_KEY ) ) {
	 
			$remote = wp_remote_get( self::LM_UPDATE_JSON_URL , array(
				'timeout' => 10,
				'headers' => array(
					'Accept' => 'application/json'
				) )
			);
	 
			if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && !empty( $remote['body'] ) ) {
				set_transient( self::LM_TRANSIENT_KEY, $remote, self::LM_TRANSIENT_TIME );
			}
	 
		}
		
		if( $remote && !is_wp_error($remote) ) {
	 
			$object = json_decode( $remote['body'] );
			
			$object->sections = (array)$object->sections ;
			$object->banners = (array)$object->banners;
			
			return $object;
	 
		}
	 
		return false;
	}
	
	
	
	
	function push_update($transient){
		
		if (empty($transient->checked))
			return $transient;
		
		
		
		if( false == $remote = get_transient(self::LM_TRANSIENT_KEY) ) {
			
			$remote = wp_remote_get( self::LM_UPDATE_JSON_URL , array(
				'timeout' => 10,
				'headers' => array(
					'Accept' => 'application/json'
				) )
			);
	 
			if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && !empty( $remote['body'] ) ) {
				set_transient( self::LM_TRANSIENT_KEY, $remote, self::LM_TRANSIENT_TIME );
			}
	 
		}
	 
		
		
		if( $remote && !is_wp_error( $remote ) ) {
			
			$remote = json_decode( $remote['body'] );
			
			if( $remote && version_compare( $this->get_plugin_data('Version',99) , $remote->version, '<' ) && version_compare($remote->requires, get_bloginfo('version'), '<' ) ) {
				$res = new stdClass();
				$res->slug = $this->get_plugin_data('LM_PLUGIN_SLUG');
				$res->plugin = $this->get_plugin_data('LM_PLUGIN');
				$res->new_version = $remote->version;
				$res->tested = $remote->tested;
				$res->package = $remote->download_link;
				
				$transient->response[$res->plugin] = $res;
				$transient->checked[$res->plugin] = $remote->version;
			}
		}

		return $transient;
	}


	
	
 
	public function after_update( $upgrader_object, $options ) {
		/* clean the cache when new plugin version is installed */
		if ( $options['action'] == 'update' && $options['type'] === 'plugin' )  {
			delete_transient( self::LM_TRANSIENT_KEY );
		}
	}
	
	
}























 
















